@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Copyright (c) 1997-1999 The Java Apache Project. All rights reserved.     *
+ * Copyright (c) 1997-2000 The Java Apache Project. All rights reserved.     *
  *                                                                           *
  * Redistribution and use in source and binary forms, with or without        *
  * modification, are permitted provided that the following conditions are    *
@@ -54,7 +54,7 @@
  * Author:      Bernard Bernstein <bernard@corp.talkcity.com>                *
  * Updated:     March 1999 Jean-Luc Rochat <jlrochat@jnix.com>               *
  * Description: solved part of fail-over problems & LB improvments           *
- * Version:     $Revision: 1.16 $
+ * Version:     $Revision: 1.21 $
  *****************************************************************************/
 
 #include "jserv.h"
@@ -70,7 +70,9 @@
 /* then so must the other. This must stay in sync with the session cookie  */
 /* or parameter set by the java code                                       */
 
-#define SESSION_IDENTIFIER "JServSessionId"
+#define SESSION_IDENTIFIER_JSERV "JServSessionId"
+#define SESSION_IDENTIFIER_TOMCAT "JSESSIONID"
+#define SESSION_IDENTIFIER_TOMCAT_PARAM "jsessionid"
 #define ROUTING_IDENTIFIER "JSERV_ROUTE"
 
 /* ========================================================================= */
@@ -90,9 +92,11 @@ get_param(char *name, request_rec *r)
   }
 
   value = strstr(r->args, pname);
+
   if (value) {
     value += strlen(pname);
     varg = value;
+    /* end of string or & */
     while (*varg && *varg != '&') {
       varg++;
       len++;
@@ -135,16 +139,55 @@ get_cookie(char *name, request_rec *r)
 /* ========================================================================= */
 /* Retrieve session id from the cookie or the parameter                      */
 /* (parameter first)                                                         */
+/* We are here able to dispatch & maintain sessions on JServ and Tomcat      */
+/*   */
+/* ========================================================================= */
 static char *
 get_jserv_sessionid(request_rec *r, char *zone)
 {
   char *val;
   char sessionid[256];
-  strcpy(sessionid, SESSION_IDENTIFIER);
-  strcat(sessionid, zone);
+
+ /* first JServ 1.1 as it is the production one */
+  strncpy(sessionid, SESSION_IDENTIFIER_JSERV, sizeof(sessionid)-1);
+ /* as our strings are defined here we know they are < 256 bytes  */
+ /* we check the routing info length */
+  if (strlen(zone) < sizeof(sessionid)-strlen(sessionid)) 
+      strcat(sessionid, zone);
+  
   val = get_param(sessionid, r);
-  if (val == NULL)
-    val = get_cookie(sessionid, r);
+
+  if (val == NULL) {
+      val = get_cookie(sessionid, r);
+
+      if (val == NULL) {
+         /* second JServ 1.0 as it is still used (no zone appended)*/
+          strcpy(sessionid, SESSION_IDENTIFIER_JSERV);
+          val = get_param(sessionid, r);
+
+          if (val == NULL) {
+              val = get_cookie(sessionid, r);
+
+              if (val == NULL) {
+                 /* 3rd Tomcat as nobody uses it in a LB production env yest */
+                 /* Tomcat using parameter */
+                  strcpy(sessionid, SESSION_IDENTIFIER_TOMCAT_PARAM);
+                  val = get_param(sessionid, r);
+
+                  if (val == NULL) {
+                     /* Tomcat using parameter (uppercase) is it useful ? */
+                     strcpy(sessionid, SESSION_IDENTIFIER_TOMCAT);
+                     val = get_param(sessionid, r);
+                  }
+                  if (val == NULL) {
+                     /* Tomcat using cookie */
+                     strcpy(sessionid, SESSION_IDENTIFIER_TOMCAT);
+                     val = get_cookie(sessionid, r);
+                  }
+              }
+          }
+      }
+  }
   return val;
 }
 
@@ -348,7 +391,21 @@ static int balance_handler(jserv_config *cfg, jserv_request *req,
     /* *** another process (watchdog process)                              */
     /* *** we just try to read in the mmap'ed file                         */
 
-#ifndef WIN32
+#if 0
+    /*
+     * We never want to do keep the current server because in the following
+     * scenario balancing will completely fail, overloading the first machine
+     * that is responding; we use simple round-robin instead like in the
+     * Win32 case.
+     *
+     * We have 10 jserv machines js1-10, and js1-5 are marked down, js6-10
+     * are ok. 50% of the requests will go to the down machines, and each
+     * one will try to find a working jserv, and ending up using js6. So
+     * we end up with a load distribution:
+     *
+     *  js6 js7 js8 js9 js10
+     *  60% 10% 10% 10% 10%
+     */
     if (req->mount->curr != req->mount->hosturls) {
       if (jserv_isdead(cfg, req->mount->hosturls)) {
         /*keep the current target */
