@@ -115,7 +115,7 @@ import java.util.zip.*;
  * @author Martin Pool
  * @author Jim Heintz
  * @author <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
- * @version $Revision: 1.15 $ $Date: 1999/12/08 20:33:25 $
+ * @version $Revision: 1.20 $ $Date: 2000/03/12 05:34:42 $
  * @see java.lang.ClassLoader
  */
 public class AdaptiveClassLoader extends ClassLoader {
@@ -138,6 +138,11 @@ public class AdaptiveClassLoader extends ClassLoader {
      */
     private Hashtable cache;
 
+    /**
+     * Save our class loader for chaining, and speed purposes.
+     */
+    private ClassLoader myParentClassLoader;
+
     /**
      * The classpath which this classloader searches for class definitions.
      * Each element of the vector should be either a directory, a .zip
@@ -194,6 +199,28 @@ public class AdaptiveClassLoader extends ClassLoader {
     public AdaptiveClassLoader(Vector classRepository)
         throws IllegalArgumentException
     {
+        this(classRepository, null);
+    }
+
+    /**
+     * Creates a new class loader that will load classes from specified
+     * class repositories.
+     *
+     * @param classRepository An set of File classes indicating
+     *        directories and/or zip/jar files. It may be empty when
+     *        only system classes are loaded.
+     * @param chainedClassLoader A class loader to attempt to load classes
+     *        as resources thru before falling back on the default system
+     *        loaders.
+     * @throw java.lang.IllegalArgumentException if the objects contained
+     *        in the vector are not a file instance or the file is not
+     *        a valid directory or a zip/jar file.
+     */
+    public AdaptiveClassLoader(Vector classRepository, ClassLoader chainedClassLoader)
+        throws IllegalArgumentException
+    {
+        myParentClassLoader = chainedClassLoader;
+
         // Create the cache of loaded classes
         cache = new Hashtable();
 
@@ -345,7 +372,7 @@ public class AdaptiveClassLoader extends ClassLoader {
      * as this one.
      */
     public AdaptiveClassLoader reinstantiate() {
-        return new AdaptiveClassLoader(repository);
+        return new AdaptiveClassLoader(repository, myParentClassLoader);
     }
 
     //------------------------------------ Implementation of Classloader
@@ -412,22 +439,27 @@ public class AdaptiveClassLoader extends ClassLoader {
         // Cache entry.
         ClassCacheEntry classCache = new ClassCacheEntry();
         while (repEnum.hasMoreElements()) {
-            byte[] classData;
+            byte[] classData=null;
 
             File file = (File) repEnum.nextElement();
-            try {
-                if (file.isDirectory()) {
-                    classData =
-                        loadClassFromDirectory(file, name, classCache);
-                } else {
-                    classData =
-                        loadClassFromZipfile(file, name, classCache);
+            File[] files=org.apache.java.io.SimpleFileFilter.fileOrFiles(file);
+            for (int i=0;i<files.length;i++)
+            {
+                file=files[i];
+                try {
+                    if (file.isDirectory()) {
+                        classData =
+                            loadClassFromDirectory(file, name, classCache);
+                    } else {
+                        classData =
+                            loadClassFromZipfile(file, name, classCache);
+                    }
+                } catch(IOException ioe) {
+                    // Error while reading in data, consider it as not found
+                    classData = null;
                 }
-            } catch(IOException ioe) {
-                // Error while reading in data, consider it as not found
-                classData = null;
             }
-    
+              
             if (classData != null) {
                 // Define the class
                 c = defineClass(name, classData, 0, classData.length);
@@ -459,6 +491,8 @@ public class AdaptiveClassLoader extends ClassLoader {
     private Class loadSystemClass(String name, boolean resolve)
         throws NoClassDefFoundError, ClassNotFoundException
     {
+        if(myParentClassLoader != null)
+            return myParentClassLoader.loadClass(name);
         Class c = findSystemClass(name);
         // Throws if not found.
 
@@ -606,7 +640,13 @@ public class AdaptiveClassLoader extends ClassLoader {
      */
     public InputStream getResourceAsStream(String name) {
         // Try to load it from the system class
-        InputStream s = getSystemResourceAsStream(name);
+        InputStream s = null;
+        if (myParentClassLoader != null) {
+            s = myParentClassLoader.getResourceAsStream(name);
+        }
+        if (s == null) {
+            s = getSystemResourceAsStream(name);
+        }
 
         if (s == null) {
             // Try to find it from every repository
@@ -617,8 +657,11 @@ public class AdaptiveClassLoader extends ClassLoader {
                     s = loadResourceFromDirectory(file, name);
                 }
                 else if(name.endsWith(".initArgs")) {
-                    File dir = new File(file.getParent());
-                    s = loadResourceFromDirectory(dir, name);
+                    String parentFile = file.getParent();
+                    if (parentFile != null) {
+                        File dir = new File(parentFile);
+                        s = loadResourceFromDirectory(dir, name);
+                    }
                 } else {
                     s = loadResourceFromZipfile(file, name);
                 }
@@ -628,7 +671,6 @@ public class AdaptiveClassLoader extends ClassLoader {
                 }
             }
         }
-
         return s;
     }
 
@@ -656,18 +698,28 @@ public class AdaptiveClassLoader extends ClassLoader {
      */
     private InputStream loadResourceFromZipfile(File file, String name) {
         ZipFile zipfile = null;
+        InputStream resourceStream = null;
         try {
             zipfile = new ZipFile(file);
             ZipEntry entry = zipfile.getEntry(name);
 
             if (entry != null) {
-                return zipfile.getInputStream(entry);
+                long length = entry.getSize();
+                resourceStream = zipfile.getInputStream(entry);
+                byte[] data = loadBytesFromStream(resourceStream, (int)length);
+                return new ByteArrayInputStream(data);
             } else {
                 return null;
             }
         } catch(IOException e) {
             return null;
         } finally {
+            if ( resourceStream != null ) {
+                try {
+                    resourceStream.close();
+                } catch ( IOException ignored ) {
+                }
+            }
             if ( zipfile != null ) {
                 try {
                     zipfile.close();
@@ -690,16 +742,23 @@ public class AdaptiveClassLoader extends ClassLoader {
      */
     public URL getResource(String name) {
 
-        // First ask the primordial class loader to fetch it from the classpath
-        URL u = getSystemResource(name);
-        if (u != null) {
-            return u;
-        }
-
         if (name == null) {
             return null;
         }
 
+        // First ask the primordial class loader to fetch it from the classpath
+
+        URL u = null;
+        if (myParentClassLoader != null) {
+            u = myParentClassLoader.getResource(name);
+        }
+        if (u == null) {
+            u = getSystemResource(name);
+        }
+        if (u != null) {
+            return u;
+        }
+
         // We got here so we have to look for the resource in our list of repository elements
         Enumeration repEnum = repository.elements();
         while (repEnum.hasMoreElements()) {
@@ -744,4 +803,19 @@ public class AdaptiveClassLoader extends ClassLoader {
         // Not found
         return null;
     }
+
+    /**
+     * Return the last modified time for a class in the 
+     * ClassCache.
+     *
+     * @throws ClassNotFoundException if class is not found
+     */
+    public long lastModified(String name) throws ClassNotFoundException {
+        ClassCacheEntry entry = (ClassCacheEntry) cache.get(name);
+        if (entry == null) {
+            throw new ClassNotFoundException("Could not find class: " + name);
+        } else {
+            return entry.lastModified;
+        }
+    }
 }
