@@ -11,8 +11,8 @@ automake
 ./configure --disable-debugging
 make dist
 
-mv ApacheJServ-1.1.tar.gz \
-$this_dir/ApacheJServ-1.1${SNAPSHOT}.tar.gz
+mv ApacheJServ-1.1.2.tar.gz \
+$this_dir/ApacheJServ-1.1.2${SNAPSHOT}.tar.gz
 
 cd $this_dir
 
